@@ -136,6 +136,11 @@ L<OAuth::Lite::AuthMethod>'s value you can choose from AUTH_HEADER, POST_BODY an
 
 The OAuth realm value for a protected-resource you wanto to access to. (optional. empty-string is set by default)
 
+=item use_request_body_hash
+
+If you use Request Body Hash extension, set 1.
+See Also L<http://oauth.googlecode.com/svn/spec/ext/body_hash/1.0/drafts/4/spec.html>
+
 =item site 
 
 The base site url of Service Provider
@@ -281,6 +286,9 @@ sub _init {
     $self->{callback_url} = $args{callback_url};
     $self->{oauth_request} = undef;
     $self->{oauth_response} = undef;
+    $self->{use_request_body_hash} = $args{use_request_body_hash} ? 1 : 0;
+    $self->{_nonce} = $args{_nonce};
+    $self->{_timestamp} = $args{_timestamp};
 }
 
 =head2 request_token_url
@@ -515,36 +523,52 @@ sub gen_oauth_request {
     my @send_data_methods = qw/POST PUT/;
     my @non_send_data_methods = qw/GET HEAD DELETE/;
 
+    my $is_send_data_method = any { $method eq $_ } @send_data_methods;
+
     my $auth_method = $self->{auth_method};
-    if (any { $method eq $_ } @non_send_data_methods) {
-        $auth_method = AUTH_HEADER
-            unless $auth_method eq URL_QUERY; 
-    } else { # POST or PUT
-        $auth_method = AUTH_HEADER
-            unless $auth_method eq POST_BODY; 
-    }
+    $auth_method = AUTH_HEADER
+        if ( !$is_send_data_method && $auth_method eq POST_BODY );
 
     if ($auth_method eq URL_QUERY) {
-        my $query = $self->gen_auth_query($method, $url, $token, $extra);
-        $url = sprintf q{%s?%s}, $url, $query;
+        if ( $is_send_data_method && !$content ) {
+            Carp::croak
+                qq(You must set content-body in case you use combination of URL_QUERY and POST/PUT http method);
+        } else {
+            if ( $is_send_data_method ) {
+                if ( my $hash = $self->build_body_hash($content) ) {
+                    $extra->{oauth_body_hash} = $hash;
+                }
+            }
+            my $query = $self->gen_auth_query($method, $url, $token, $extra);
+            $url = sprintf q{%s?%s}, $url, $query;
+        }
     } elsif ($auth_method eq POST_BODY) {
         my $query = $self->gen_auth_query($method, $url, $token, $extra);
         $content = $query;
+        $headers->header('Content-Type', q{application/x-www-form-urlencoded});
     } else {
-        my $header = $self->gen_auth_header($method, $url,
-            { realm => $realm, token => $token, extra => $extra });
-        $headers->header( Authorization => $header );
-        if (keys %$extra > 0) {
+        my $origin_url = $url;
+        warn "auth header";
+        if ( keys %$extra > 0 ) {
             my $data = normalize_params($extra);
-            #my $data = join('&', @extra_pairs);
-            if (any { $method eq $_ } @send_data_methods) {
+            if ( $is_send_data_method && !$content ) {
                 $content = $data;
             } else {
                 $url = sprintf q{%s?%s}, $url, $data;
             }
         }
+        if ( $is_send_data_method ) {
+            warn "is_send";
+            if ( my $hash = $self->build_body_hash($content) ) {
+                warn "hash";
+                $extra->{oauth_body_hash} = $hash;
+            }
+        }
+        my $header = $self->gen_auth_header($method, $origin_url,
+            { realm => $realm, token => $token, extra => $extra });
+        $headers->header( Authorization => $header );
     }
-    if (any { $method eq $_ } @send_data_methods) {
+    if ( $is_send_data_method ) {
         $headers->header('Content-Type', q{application/x-www-form-urlencoded})
             unless $headers->header('Content-Type');
         $headers->header('Content-Length', bytes::length($content) );
@@ -660,7 +684,7 @@ sub gen_auth_header {
     my $extra = $args->{extra} || {};
     my $params = $self->gen_auth_params($method, $url, $args->{token}, $extra);
     my $realm = $args->{realm} || '';
-    my $authorization_header = build_auth_header($realm, $params);
+    my $authorization_header = build_auth_header($realm, {%$params, %$extra});
     $authorization_header;
 }
 
@@ -709,8 +733,8 @@ sub gen_auth_params {
     my $params = {};
     $extra ||= {};
     $params->{oauth_consumer_key} = $self->consumer_key || '';
-    $params->{oauth_timestamp} = time();
-    $params->{oauth_nonce} = gen_random_key();
+    $params->{oauth_timestamp} = $self->{_timestamp} || time();
+    $params->{oauth_nonce} = $self->{_nonce} || gen_random_key();
     $params->{oauth_version} = $OAuth::Lite::OAUTH_DEFAULT_VERSION;
     my $token_secret = '';
     if (defined $token) {
@@ -770,6 +794,24 @@ sub oauth_clear {
     $self->{oauth_response} = undef;
 }
 
+=head2 build_body_hash
+
+Build body hash according to the spec for 'OAuth Request Body Hash extension'
+http://oauth.googlecode.com/svn/spec/ext/body_hash/1.0/drafts/4/spec.html
+
+    my $hash = $self->build_body_hash($content);
+
+=cut
+
+sub build_body_hash {
+    my ( $self, $content ) = @_;
+    if ( $self->{use_request_body_hash} ) {
+        my $hash = $self->{signature_method}->build_body_hash($content);
+        return $hash;
+    }
+    return;
+}
+
 =head1 AUTHOR
 
 Lyo Kato, C<lyo.kato _at_ gmail.com>
